<?php foreach ($object->customvars as $name => $value): ?>
<tr>
    <th><?= $this->escape($name) ?></th>
    <td class="custom-variables"><?= $this->customvar($value) ?></td>
</tr>
<?php endforeach ?>
