<div id="login">
    <div class="login-wrapper">
        <div class="login-form centered-ghost" data-base-target="layout">
            <div class="login-form-content centered-content">
                <div role="status" class="sr-only">
                    <?= $this->translate(
                        'Welcome to Icinga Web 2. For users of the screen reader Jaws full and expectant compliant'
                        . ' accessibility is possible only with use of the Firefox browser. VoiceOver on Mac OS X is tested on'
                        . ' Chrome, Safari and Firefox.'
                    ) ?>
                </div>
                <div class="logo-wrapper"><div id="icinga-logo" class="invert" aria-hidden="true"></div></div>
                <?php if ($requiresSetup): ?>
                    <p class="config-note"><?= sprintf(
                        $this->translate(
                            'It appears that you did not configure Icinga Web 2 yet so it\'s not possible to log in without any defined '
                            . 'authentication method. Please define a authentication method by following the instructions in the'
                            . ' %1$sdocumentation%3$s or by using our %2$sweb-based setup-wizard%3$s.'
                        ),
                        '<a href="https://icinga.com/docs/" title="' . $this->translate('Icinga Web 2 Documentation') . '">', // TODO: More exact documentation link
                        '<a href="' . $this->href('setup') . '" title="' . $this->translate('Icinga Web 2 Setup-Wizard') . '">',
                        '</a>'
                    ) ?></p>
                <?php endif ?>
                <?= $this->form ?>
                <div id="login-footer">
                    <p>Icinga Web 2 &copy; 2013-<?= date('Y') ?></p>
                    <?= $this->qlink($this->translate('icinga.com'), 'https://icinga.com') ?>
                </div>
            </div>
        </div>
        <ul id="social">
            <li>
            <?= $this->qlink(
                    null,
                    'https://twitter.com/icinga',
                    null,
                    array(
                        'target'  => '_blank',
                        'icon'    => 'twitter',
                        'title'   => $this->translate('Icinga on Twitter')
                    )
                ) ?>
            </li>
            <li>
                <?= $this->qlink(
                    null,
                    'https://www.facebook.com/icinga',
                    null,
                    array(
                        'target'  => '_blank',
                        'icon'     => 'facebook-squared',
                        'title'    => $this->translate('Icinga on Facebook')
                    )
                ) ?>
            </li>
            <li><?= $this->qlink(
                    null,
                    'https://github.com/Icinga',
                    null,
                    array(
                        'target'    => '_blank',
                        'icon'      => 'github-circled',
                        'title'     => $this->translate('Icinga on GitHub')
                    )
                ) ?>
            </li>
        </ul>
    </div>
</div>
