<?php if (! $this->compact && ! $hideControls): ?>
<div class="controls">
    <?= $tabs ?>
</div>
<?php endif ?>
<div class="content">
<?php
if (isset($stackTraces)) {
    foreach ($messages as $i => $message) {
        echo '<p tabindex="-1" class="autofocus error-message">' . nl2br($this->escape($message)) . '</p>'
            . '<hr>'
            . '<pre>' . $this->escape($stackTraces[$i]) . '</pre>';
    }
} else {
    foreach ($messages as $message) {
        echo '<p tabindex="-1" class="autofocus error-message">' . nl2br($this->escape($message)) . '</p>';
    }
}
?>
</div>
